
library(ape)
library(phangorn)
library(stringr)


source("C:/Users/GRJ/AAA/Programming/Biology/DENIMSimulations/Utils/scenario-dir-names.R")
source("C:/Users/GRJ/AAA/Programming/Biology/DENIMSimulations/Utils/mcoal-tree-to-Newick.R")





# Adds an edge to a rooted tree at its root. 
# This makes it into an unrooted tree, with one more tip labelled "origin"
add.origin.to.rooted.tree <- function(tree) {
	stopifnot(is.rooted(tree))
	rooti <- getRoot(tree)
	ntips <- length(tree$tip.label)
	utree <- tree
	w <- which(tree$edge > ntips)
	utree$edge[w] <- tree$edge[w] + 1
	utree$edge <- rbind(utree$edge, c(rooti+1, ntips+1))
	utree$edge.length <- c(utree$edge.length, 0)
	utree$tip.label <- c(utree$tip.label, "origin")
	utree$Nnode <- utree$Nnode + 1
	utree
}



rooted.KF.dist <- function(tree1, tree2) {
	KF.dist(add.origin.to.rooted.tree(tree1), add.origin.to.rooted.tree(tree2))
}





true.phylo <- function(sdir, rep) {
	ctrlfpath <- paste0(beast.dir.from.scenario.dir.and.rep(sdir, rep), "/ctrl.txt")
	newick <- mcctr.nwk.mc.file.to.newick(ctrlfpath)
	read.tree(text=newick)
}




mcmc.treelist <- function(sdir, rep) {
	tree.fpath <- paste0(beast.dir.from.scenario.dir.and.rep(sdir, rep), "/species.trees")
	treelist <- read.nexus(tree.fpath, force.multi = TRUE)
  treelist <- treelist[-(1:burnin.for.MCMC)]
}


# Not used(?)
true.treelength <- function(true.phy) {
  nodeheights <- sort(branching.times(true.phy))
  treelength <- sum(nodeheights) + nodeheights[length(nodeheights)]
  treelength
}



make.mean.kfdists <- function(scenario.directories) {
	posterior.mean.kfdists <-
		matrix(0,
					 ncol = length(scenario.directories),
					 nrow = number.of.replicates)
	colnames(posterior.mean.kfdists) <- scenario.directories
	
	for (sdir in scenario.directories) {
		cat(sdir, "... ")
		for (rep in first.replicate:last.replicate) {
			true.phy <- true.phylo(sdir, rep)
			treelist <- mcmc.treelist(sdir, rep)
			
			kfdists <- rep(0, length(treelist))
			for (i in 1:length(treelist)) {
				kfdists[i] <- rooted.KF.dist(treelist[[i]], true.phy)
			}

			posterior.mean.kfdists[rep, sdir] <-  mean(kfdists)
		}
		cat("Done mean distance to true tree\n")
	}
	posterior.mean.kfdists
}







#######################################################################################






plot.boxplot <- function(posterior.mean.kfdists, sdirs, reorder) {
	stopifnot(length(sdirs) == length(reorder))
	par(mfcol = c(1, 1))
	par(mar = c(9, 3, 4, 2) + 0.1)
	
	boxplot(posterior.mean.kfdists[, reorder], xaxt = "n", 
					main = "Posterior mean of branch scores")
	axis(1,
			 at = 1:length(sdirs),
			 labels = labels.for.plot(sdirs[reorder]),
			 las = 3)
}


plot.boxplot.screen.and.pdf <- function(posterior.mean.kfdists, sdirs, reorder) {
	plot.boxplot(posterior.mean.kfdists, sdirs, reorder)
	pdf(file=fpath.for.branch.scores(), pointsize=10, width=6, height=8)
	plot.boxplot(posterior.mean.kfdists, sdirs, reorder)
	dev.off()
	}



####################################################################################

do.branch.scores.spp.4 <- function() {
	posterior.mean.kfdists <- make.mean.kfdists(fourspp.all.scenario.dirs)

	reorder <- c(13, 1:12, 14:17)
	
	plot.boxplot.screen.and.pdf(posterior.mean.kfdists, fourspp.all.scenario.dirs, reorder)
	
	# par(mfcol = c(1, 1))
	# par(mar = c(10, 4, 4, 2) + 0.1)
	# boxplot(posterior.mean.kfdists[, reorder], xaxt = "n", #ylim=c(0,0.7),
	# 				main = "Posterior mean of branch scores")
	# axis(1,
	# 		 at = 1:17,
	# 		 labels = fourspp.all.scenario.dirs[reorder],
	# 		 las = 3)
	
	
}


#################################################################################################




do.branch.scores.spp.10  <- function() {
	nnn <- 0
	if (just.coal.paraphyly) { nnn <- nnn + 1 }
	if (just.leache) { nnn <- nnn + 1 }
	if (just.coal.big.rates) { nnn <- nnn + 1 }
	stopifnot(nnn <= 1)
	
	if (just.coal.paraphyly) {
		posterior.mean.kfdists <- make.mean.kfdists(tenspp.just.coal.paraphyly.dirs)
		reorder <- c(5,1,2,3,4,6,7)
		plot.boxplot.screen.and.pdf(posterior.mean.kfdists, tenspp.just.coal.paraphyly.dirs, reorder)
	} else if (just.leache) {
		posterior.mean.kfdists <- make.mean.kfdists(tenspp.leache.scenario.dirs)
		reorder <- 1:19
		plot.boxplot.screen.and.pdf(posterior.mean.kfdists, tenspp.leache.scenario.dirs, reorder)
	} else if (just.coal.big.rates) {
		sdirs <- c("COAL005", "COAL01", "COAL02", "COAL04", "COAL08", "COAL16")
		reorder <- 1:6
		posterior.mean.kfdists <- make.mean.kfdists(sdirs)
		plot.boxplot.screen.and.pdf(posterior.mean.kfdists, sdirs, reorder)
	} else {
		posterior.mean.kfdists <- make.mean.kfdists(tenspp.all.scenario.dirs)
		reorder <- c(19, 1:18, 20:25)
		plot.boxplot.screen.and.pdf(posterior.mean.kfdists, tenspp.all.scenario.dirs, reorder)

	}
}






####################################################################################################


do.branch.scores <- function(nspp) {
	if (nspp == 10) {
		do.branch.scores.spp.10()
	}
	if (nspp == 4) {
		do.branch.scores.spp.4()
	}
}

